// Database.cpp: implementation of the CDatabase class.
//
//////////////////////////////////////////////////////////////////////

#include "DatabaseEntry.h"
#include "Database.h"
#include <string.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDatabase::CDatabase(int nMaxEntries)
{
m_nNumberEntries = 0;
m_nDatabaseSize = 0;

m_pEntries = new CDatabaseEntry[ nMaxEntries ];

if (m_pEntries)
	m_nDatabaseSize = nMaxEntries;


}

CDatabase::~CDatabase()
{
if (m_pEntries)
	delete []m_pEntries;
}

////////////////////////////////////////////////////////////////
//
// Method: SearchByName( szKey )
//
// Searches the datbase for a NAME field that matches the
// key passed. Case insensitive.
//
// Returns 0 on failure, or a logical item number, 1 to N.
//
// Note: Logical Item Number = Physical Item Number + 1
//
////////////////////////////////////////////////////////////////


int CDatabase::SearchByName(char* szKey)
{
int i;

if (m_nNumberEntries == 0)
		return 0;		// no match


for(i=0;i<m_nNumberEntries;i++)
	{
	if (strcmpi( szKey, m_pEntries[i].GetItemName() ) == 0)
			return(i+1);

	}

return 0;	// no match found
}

////////////////////////////////////////////////////////////////
//
// Method: PrintEntry( nItemID)
//
// nItemID is the physical index (0 to N-1) of the object to
// be printed. The output goes to the console.
//
//
////////////////////////////////////////////////////////////////

void CDatabase::PrintEntry(int nItemID)
{
if (nItemID >= m_nNumberEntries)
	return;

m_pEntries[nItemID].PrintContents();
}


////////////////////////////////////////////////////////////////
//
// Method: DeleteItem( nItemID)
//
// nItemID is the physical index ( 0 to N-1) of the object to
// delete. The database is automatically defragmented, ala
// Microsloft, as needed.
// 
//
////////////////////////////////////////////////////////////////



void CDatabase::DeleteItem(int nItemID)
{
int i;

if (nItemID >= m_nNumberEntries) return;	// invalid item #

//
// if item is last one in database, just decrement item
// counter. Otherwise, must shuffle the remaining items
// downward to fill the vacancy.
//

if (nItemID == (m_nNumberEntries-1) )
	{
	m_nNumberEntries--;
	return;
	}

//
// Deleting this item will leave a hole, so shuffle the
// remaining items downward (very inefficient way of doing
// this, but hey, we're Microsloft!)
//

for(i=nItemID; i< (m_nNumberEntries-1) ; i++)
	m_pEntries[i] = m_pEntries[i+1];

m_nNumberEntries--;

}



////////////////////////////////////////////////////////////////
//
// Method: GetNumberEntries
//
// Returns: The actual number of entries in the database, or
// 0 if there are none.
//
////////////////////////////////////////////////////////////////

int CDatabase::GetNumberEntries()
{
return m_nNumberEntries;
}


////////////////////////////////////////////////////////////////
//
// Method: GetDataBaseSize
//
// Returns: The maximum number of entries that the database can
// hold.
//
////////////////////////////////////////////////////////////////


int CDatabase::GetDatabaseSize()
{
return m_nDatabaseSize;
}

////////////////////////////////////////////////////////////////
//
// Method: Add entry -- Returns -1 on failure, or the new number of
// items in the database.
//
////////////////////////////////////////////////////////////////

int CDatabase::AddEntry(char * szItemName, double dPrice, int nPartNumber)
{

if (m_nNumberEntries >= m_nDatabaseSize)
	return -1;	// error, db full


m_pEntries[m_nNumberEntries].SetItemName( szItemName );
m_pEntries[m_nNumberEntries].SetItemCost( dPrice );
m_pEntries[m_nNumberEntries].SetPartNumber( nPartNumber );

m_nNumberEntries++;

return(m_nNumberEntries);

}
