#include <stdio.h>
#include "databaseEntry.h"
#include "database.h"
#include "Input.h"
#include <stdlib.h>
#include <conio.h>




////////////////////////////////////////////////////////////////
//
// main.cpp
//
// Test framework for the DATABASE project.
//
// Version 1.0
// (c) 2000 Tom A. Wheeler
//
// This program uses two classes, CDatabase (supplied by the
// instructor) and CDatabaseEntry (written by the student) to
// implement a database using OOP techniques.
//
// Operation:
//
// The main program instantiates a CDatabase object with the
// requested number of records (entries). The CDatabase object
// in turn creates and manages the required number of
// CDatabaseEntry objects. The interface to the database
// object is contained entirely within CDatabase -- there are
// no global variables or other parameters of any type
// necessary.
//
// This program is a minimum framework for testing the CDatabase
// object.
//
//
////////////////////////////////////////////////////////////////










////////////////////////////////////////////////////////////////
//
// Command routine: AddEntry()
//
// Carries out the dialog for adding an entry to the
// database object.
//
////////////////////////////////////////////////////////////////


void AddEntry(CDatabase* pDbase)
{
char Name[128];
int PartNumb;
double Price;
int nResult;
CInput Input;	// general purpose input object

//
// Input the various data fields
//

Input.Input( Name,80, "What is the item name?");
Input.Input( Price, "What is the price?");
Input.Input( PartNumb, "What is the part number?");

//
// See if the datbase can accept our data
//

nResult = pDbase->AddEntry( Name, Price, PartNumb );

if (nResult < 0)
	printf("\7The database did not accept this item.");

}




////////////////////////////////////////////////////////////////
//
// Command routine: DeleteEntry()
//
// Carries out the necessary dialog for deleting a record
// from the database object.
//
////////////////////////////////////////////////////////////////



void DeleteEntry(CDatabase* pDbase )
{
int nCount;
int nWhich;
char cKey;
CInput Input;

nCount = pDbase->GetNumberEntries();

if (nCount == 0)
	{
	printf("\nThe database is empty!\n");
	return;
	}


//
// Input with dynamic prompt message
//
printf("Which item to delete, 1- %d?", nCount );
Input.Input(nWhich);

if (nWhich > nCount)
	{
	printf("Invalid item number.\n");
	return;
	}

pDbase->PrintEntry( nWhich -1 );
printf("Delete this one (Y/N)? ");
fflush(stdin);
cKey = getche();

if (toupper(cKey) == 'Y')
	{
	pDbase->DeleteItem(nWhich-1);
	printf("Item deleted.\n");
	}

}








////////////////////////////////////////////////////////////////
//
// Command routine: Browse()
//
// This implements the database browser dialog.
//
//
////////////////////////////////////////////////////////////////


void BrowseDB(CDatabase* pDbase)
{
int nWhich;
char cKey;
int nCount;

if ((nCount=pDbase->GetNumberEntries()) == 0)
	{
	printf("\7\nDatabase is empty, can't browse.\n");
	return;
	}

nWhich = 0;

for(;;)
	{
	printf("\nItem number %d", nWhich + 1 );
	pDbase->PrintEntry(nWhich);
	printf("(+) Proceed, (-) Back, (D)elete, (Q)uit?");
	cKey = toupper(getche());
	printf("\n\n");

	if (cKey == 'Q') return;

	if ((cKey == '+') && (nWhich < (nCount -1) ))
		{
		nWhich++;
		continue;
		}

	if ((cKey == '-') && (nWhich) )
		{
		nWhich--;
		continue;
		}

	if (cKey == 'D')
		{
		printf("\nDelete this item (Y/N)?");
		if (toupper(getche())=='Y')
			{
			pDbase->DeleteItem(nWhich);
			nCount=pDbase->GetNumberEntries();


			if (nCount==0)
				{
				printf("\7\nThe database is now empty. Exiting browse.\n");
				return;
				}

//
// If we deleted the last record, move record pointer backwards
// by one position so that a valid record continues to display.
//

			if (nWhich > (nCount-1))
				nWhich = nCount-1;

			}
			else
			{
			printf("\nItem NOT deleted.\n");
			continue;
			}
		}
	}

}


////////////////////////////////////////////////////////////////
//
// Command routine: SearchItem()
//
// This implements the dialog for searching the database.
//

////////////////////////////////////////////////////////////////

void SearchItem(CDatabase* pDbase)
{
char szKey[128];
int nResult;
CInput Input;

Input.Input(szKey, 80, "What name to search for?");


nResult = pDbase->SearchByName( szKey );

if (nResult == 0)
	{
	printf("\n\7The item \"%s\" was NOT found.\n", szKey );
	return;
	}

printf("Item %d\n", nResult);
pDbase->PrintEntry( nResult - 1 );

}


////////////////////////////////////////////////////////////////
//
// Main -- Mainline executive program.
//
// Main instantiates the database object, and prints a
// menu of choices for the user. The various dialogs are
// called depending upon the user's choice.
//
////////////////////////////////////////////////////////////////

void main(void)
{
CDatabase* pDB;
int nChoice;
CInput Input;	// general-purpose input object


Input.Input(nChoice , "How many entries will be in dbase?");

//
// Instantiate a database with the selected number
// of records
//

pDB = new CDatabase(nChoice);

if (pDB == NULL)
	{
	printf("Could not instantiate database.");
	return;
	}


//
// Menu Loop
//

for(;;)
	{
	printf("\nDatabase size: %d, Items in database: %d\n\n",
	pDB->GetDatabaseSize(), pDB->GetNumberEntries() );


	printf("1. Add entry to database\n");
	printf("2. Delete entry from database\n");
	printf("3. Browse database\n");
	printf("4. Search for item by name\n");
	printf("5. Exit program.\n\nChoose?");

	Input.Input(nChoice);

	switch( nChoice)
		{
		case 1:	AddEntry(pDB);
				break;

		case 2:	DeleteEntry(pDB);
				break;

		case 3:	BrowseDB(pDB);
				break;

		case 4:	SearchItem(pDB);
				break;

		case 5: return;
		}
	}
}

